/**
 * A builder that carries a String value representing a list with a particular
 * delimiter.
 */
public class CSVStringBuilder extends SimpleStringBuilder {
  // THIS IS AN EXAMPLE OF THE DECORATOR PATTERN

  // AF(this) = super.AF(this), also having a delimiter this.delimiter,
  //            representing a list of size 0 if this.hasAppended is false, and
  //            representing a list delimited by this.delimiter otherwise.
  // RI: true

  /** The delimiter that this uses to separate elements of the list. */
  private String delimiter;
  /** If false, no elements have been appended to the list. */
  private boolean hasAppended;

  public CSVStringBuilder(String delimiter) {
    this.delimiter = delimiter;
    this.hasAppended = false;
  }

  /**
   * @param target A string to be appended to the list of this.
   * @spec.requires target != null
   * @spec.modifies this
   * @spec.effects target is appended to the list of this.
   */
  @Override
  public void append(String target) {
    // Append target to the list, delimiting by this.delimiter.  Make sure there
    // are no extraneous delimiters at the beginning or end of the list.
    // Delegate any work possible to super.
    // TODO
  }

  /**
   * @param target An array of chars to be appended to the list of this.
   * @param count The number of characters to append from target.
   * @spec.requires target != null, 0 <= count <= target.length
   * @spec.modifies this
   * @spec.effects the first count chars from target are appended to
   *               the end of the list of this.
   */
  @Override
  public void append(char[] target, int count) {
    // Append target to the list, delimiting by this.delimiter.  Make sure there
    // are no extraneous delimiters at the beginning or end of the list.
    // Delegate any work possible to super.
    // TODO
  }
}
