/**
 * Retrieves and writes file contents.  A FileManager does not necessarily write
 * to files immediately, but may store their intended state in a cache until
 * prompted to update the file system with changes.
 */
public interface FileManager {

  /**
   * @param file The filepath of a file to retrieve the contents of.
   * @spec.requires file != null
   * @return null if file does not point to a valid file, or if an error
   *         occured while reading the file.
   * @return the contents of the file pointed at by file otherwise.  The
   *         contents may be the current contents of the file or the contents
   *         written to this manager but not updated in the file system.
   */
  public String get(String file);

  /**
   * @param file The filepath of a file to write to.
   * @param contents The new contents of the file.
   * @spec.requires file, contents != null
   * @spec.modifies this
   * @spec.effects writes contents to the given file.  The changes are not
   *               guaranteed to take effect on the filesystem.
   */
  public void put(String file, String contents);

  /**
   * @param file The filepath of a file to ensure has updated contents.
   * @spec.requires file != null
   * @spec.modifies this
   * @spec.effects Updates the file with any changes given to this previously,
   *               if any, and if the given file can be written to.
   * @returns true if the given file's contents are guaranteed to be updated.
   */
  public boolean update(String file);
}
