import java.util.*;
import java.io.*;

public class NonCachingFileManager implements FileManager {
  // THIS IS AN EXAMPLE OF THE ADAPTER PATTERN.

  // AF(this) = A file manager with files always updated fully, except for the
  //            files in this.errorSet.
  // RI: this.errorSet != null && contains no nulls

  /** Performs extensive representation invariant checks when true. */
  private static final boolean DEBUG = true;

  /** The paths to files that are failed to write to. */
  private Set<String> errorSet;

  /**
   * @spec.effects Constructs a NonCachingFileManager with no cache.
   */
  public NonCachingFileManager() {
    errorSet = new HashSet<>();

    checkRep();
  }

  @Override
  public String get(String file) {
    checkRep();
    assert file != null : "Illegal null argument.";

    SimpleStringBuilder res = new SimpleStringBuilder();

    // Read the file into the string builder.  Return null on any errors.
    // TODO

    try {
      return res.toString();
    } finally {
      checkRep();
    }
  }

  @Override
  public void put(String file, String contents) {
    checkRep();
    assert file != null : "Illegal null argument.";
    assert contents != null : "Illegal null argument.";

    // Write contents into the file on the file system.  On error, add the file
    // to this.errorSet.
    // TODO

    checkRep();
  }

  @Override
  public boolean update(String file) {
    checkRep();
    assert file != null : "Illegal null argument.";

    // If the file returned an error on put, this would return false for
    // SimpleFileManager.  Otherwise, we are fine to return true since we always
    // update the file system immediately.
    try {
      return !errorSet.contains(file);
    } finally {
      checkRep();
    }
  }

  /**
   * Checks that the representation invariant holds.
   */
  private void checkRep() {
    assert this.errorSet != null;
    if (DEBUG) {
      assert !this.errorSet.contains(null);
    }
  }
}
