/**
 * A builder that carries a String value.
 */
public class SimpleStringBuilder {
  // THIS IS AN EXAMPLE OF THE BUILDER PATTERN.

  // AF(this) = A builder with a value of (new String(this.val, 0, this.size))
  // RI: this.val != null, 0 <= this.size <= this.val.length

  /** The default initial size of val. */
  private static final int INITIAL_CAPACITY = 1024;

  /** Contains the characters of the value of this. */
  private char[] val;
  /** Denotes the length of the value of this. */
  private int size;

  /**
   * @spec.effects Constructs a new builder with the value of an empty string.
   */
  public SimpleStringBuilder() {
    this(INITIAL_CAPACITY);
    checkRep();
  }

  /**
   * @param capacity The number of chars that can reasonably be expected to
   *                 be in Strings added to this.
   * @spec.requires capacity >= 0
   * @spec.effects Constructs a new builder with the value of an empty string.
   */
  public SimpleStringBuilder(int capacity) {
    assert capacity >= 0 : "Illegal negative argument.";

    // Initialize the initial character array and its conceptual size.
    // TODO

    checkRep();
  }

  /**
   * @param target A String to be added to this.
   * @spec.requires target != null
   * @spec.modifies this
   * @spec.effects target is appended to the end of the value of this.
   */
  public void append(String target) {
    checkRep();
    assert target != null : "Illegal null argument.";

    // Resize the character array to fit the new String if necessary.
    // TODO (hint: see private helper)

    // Add the characters of target to the end of the builder.
    // TODO

    checkRep();
  }

  /**
   * @param target An array of chars to be added to this.
   * @param count The number of characters to append from target.
   * @spec.requires target != null, 0 <= count <= target.length
   * @spec.modifies this
   * @spec.effects the first count chars from target are appended
   *               to the end of the value of this.
   */
  public void append(char[] target, int count) {
    checkRep();
    assert target != null : "Illegal null argument.";
    assert count >= 0 : "Illegal negative argument.";
    assert count <= target.length : "Illegal large argument.";

    // Resize the character array to fit the new String if necessary.
    // TODO

    // Add the characters of target to the end of the builder.
    // TODO

    checkRep();
  }

  /**
   * @return the value of this.
   */
  public String toString() {
    checkRep();

    try {
      // Return the value of the builder.
      // TODO
      return null;
    } finally {
      checkRep();
    }
  }

  /**
   * Updates the size of val so that it is larger than this.size.
   */
  private void resize() {
    // Compute a new size for the array by doubling the size until it's enough
    // to fit the String.
    int newSize;
    // TODO

    char[] newVal = new char[newSize];
    // Copy the old value of the builder to the new array.
    // TODO

    // Set the builder to use the new, larger array.
    // TODO
  }

  /**
   * Checks that the representation invariant holds.
   */
  private void checkRep() {
    assert this.val != null;
    assert this.size >= 0;
    assert this.size <= this.val.length;
  }
}
